pub struct ProgramExit {
pub message: String,
pub exit_status: i32,
pub styled_message: Option<String>,
pub kind: ExitKind,
}Expand description
DTO struct representing an early or abrupt program exit.
Fields
message: StringThe message linked to the program exit.
exit_status: i32The exit status code to be returned by the script.
styled_message: Option<String>The ANSI-styled message linked to the program exit.
Same as ProgramExit::message but styled.
kind: ExitKindThe kind of program exit.