Crate gitignore_template_generator

Components used to safely generate gitignore templates from a template generator API over HTTP.

Modules

constant
Define globally-shared constants.
helper
Define components to help in other modules.
http_client
Define components to make HTTP calls.
parser
Define components to parse cli args.
validator
Define components to validate cli args.

Structs

GitignoreTemplateManager
Manager of gitignore templates.
ProgramExit
DTO struct representing an early or abrupt program exit.

Enums

ExitKind
Enum for kind of program exit.

Traits

TemplateGenerator
Template generator trait to generate a template via an API call made over HTTP.
TemplateLister
Template lister trait to list available templates via an API call made over HTTP.