Struct Args

pub struct Args {
    pub template_names: Vec<String>,
    pub server_url: String,
    pub endpoint_uri: String,
    pub show_help: bool,
    pub show_version: bool,
    pub show_author: bool,
}
Struct to gather cli args parsing result.

Used by crate::parser::ArgsParser implementations to store parsing result.

Fields

template_names: Vec<String>

A non-empty list of gitignore template names.

  • Represented by the provided positional arguments, and required unless any of author, version or help options are given.
  • This field does not allow commas in any of its field.
server_url: String

The gitignore template generator service url.

endpoint_uri: String

The gitignore template generator service endpoint uri.

show_help: bool

The boolean indicator of whether to display help infos or not.

  • Optional value represented by the cli option crate::constant::cli_options::HELP, and falling back to false if not provided in cli args.
  • Has precedence over version and author options if multiple are given
show_version: bool

The boolean indicator of whether to display version infos or not.

  • Optional value represented by the cli option crate::constant::cli_options::VERSION, and falling back to false if not provided in cli args.
  • Has precedence over author option if multiple are given
show_author: bool

The boolean indicator of whether to display author infos or not.

Implementations

Source

impl Args

pub fn with_template_names(self, template_names: Vec<String>) -> Self

Sets new value for template_names field.

It needs to be called on struct instance and effectively mutates it.

Arguments
  • template_names - The new value to be assigned to template_names field.
Returns

The mutated borrowed instance.

pub fn with_server_url(self, server_url: &str) -> Self

Sets new value for server_url field.

It needs to be called on struct instance and effectively mutates it.

Arguments
  • server_url - The new value to be assigned to server_url field.
Returns

The mutated borrowed instance.

pub fn with_endpoint_uri(self, endpoint_uri: &str) -> Self

Sets new value for endpoint_uri field.

It needs to be called on struct instance and effectively mutates it.

Arguments
  • endpoint_uri - The new value to be assigned to endpoint_uri field.
Returns

The mutated borrowed instance.